from django import forms


class SetRequiredMixin:
    """ mixin class that sets form fields as required or not_required based on the form Meta class parameters
     'required' and/or 'not_required'. Both parameters are a list of fields in the form. The 'required' parameter takes
     precedence over 'not_required' parameter if fields are listed in both lists.

    example:

        class MyModelForm(SetRequiredMixin, forms.ModelForm):

            class Meta:
                model = MyModel
                widgets = {
                    'some_field': forms.TextInput(attrs={'class': 'form-control'}),
                    'some_other_field': forms.TextInput(attrs={'class': 'form-control'}),
                }
                required = ['some_field']
                not_required = ['some_other_field']
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if getattr(self.Meta, 'not_required', None):
            for field in self.Meta.not_required:
                self.fields[field].required = False

        if getattr(self.Meta, 'required', None):
            for field in self.Meta.required:
                self.fields[field].required = True



from django.views.generic import View
from django.utils import timezone
from django.shortcuts import render
from django.conf import settings
import datetime
import calendar
from dateutil.rrule import rrule, MONTHLY


def get_color_list():
    """ return a list of color variables, based on bootstrap theme definitions, to use in report templates """
    return ['theme.primary', 'theme.success', 'theme.secondary', 'theme.info', 'theme.warning',
            'theme.danger', 'theme.dark', 'theme.light', 'theme.blue', 'theme.green', 'theme.red', 'theme.orange',
            'theme.yellow', 'theme.purple', 'theme.teal', 'theme.cyan', 'theme.indigo', 'theme.pink', 'theme.white',
            'theme.gray', 'theme.gray-dark']


def get_colors(count):
    """ return a list of `count` colors """
    color_list = get_color_list()
    return_list = list()
    color = 0
    for i in range(count):
        return_list.append(color_list[color])
        if color >= len(color_list) - 1:
            color = 0
        color += 1
    return return_list


def get_annual_timestamps(start=None, end=None, reverse=False):
    """
    Get a list of timestamps to use in annual line charts

    Args:
        start: (datetime or timezone) time to generate timestamps from (defaults to a year ago)
        end: (datetime or timezone)  time to generate timestamps to (defaults to now)
        reverse: (bool) if True  - reverse the list and use start of the month for timestamps
                        if False - use end of the month for timestamps

    Returns:
        list of timestamps
    """
    if not end:
        end = timezone.now()
    if not start:
        start = end - datetime.timedelta(365)
    if reverse:
        data = [d.replace(day=1) for d in rrule(MONTHLY, dtstart=start, until=end)]
    else:
        data = [d.replace(day=calendar.monthrange(d.year, d.month)[1])
                for d in rrule(MONTHLY, dtstart=start, until=end)]
    if reverse:
        data.reverse()
    return data


def get_timestamps(now=None):
    """
    return a tuple of timestamps; from a point in time (now) for the past day, week, month, and year

    Args:
        now: (datetime or timezone) initial timestamp used to determine day, week, month, year date diffs

    Returns:
        tuple of timestamps
    """
    if not now:
        now = timezone.now()
    last_day = now - datetime.timedelta(days=1)
    last_week = now - datetime.timedelta(days=7)
    last_month = now - datetime.timedelta(days=365.2425 / 12)
    last_year = now - datetime.timedelta(days=365.2425)
    return last_day, last_week, last_month, last_year


def get_dated_data(queryset, ts_lookup):
    """
    return queryset of results filtered by the past day, week, month, and year

    Args:
        queryset: (django queryset) django queryset used in gathering day, week, month, year lookups
        ts_lookup: (str) timestamp and expression to use in filter ex. (created_at__gte)

    Returns:
        tuple of querysets filtered by the past day, week, month, and year
    """
    last_day, last_week, last_month, last_year = get_timestamps()
    data_day = queryset.filter(**{ts_lookup: last_day})
    data_week = queryset.filter(**{ts_lookup: last_week})
    data_month = queryset.filter(**{ts_lookup: last_month})
    data_year = queryset.filter(**{ts_lookup: last_year})
    return data_day, data_week, data_month, data_year


def build_annual_progress_chart(dataset_list):
    """
    process a list of datasets and return data required to plot an annual progress chart

    Args:
        dataset_list - (list of dictionaries) set of data to display; list of dictionaries containing the following:
                       title     - title to display for dataset
                       queryset  - queryset to use in counts
                       dt_field  - datetime field in model

                       example:
                           [{title='Owners', queryset=Owner.objects.all(), dt_field='created_at'}, ...]

    Returns:
        tuple containing:
            list of month labels
            list of month timestamps
            list of dictionaries containing data needed to plot chart
    """
    return_dataset_list = list()
    color_list = get_color_list()
    annual_timestamp_list = get_annual_timestamps()
    month_labels = [ts.strftime('%B') for ts in annual_timestamp_list]

    color = 0
    for dataset in dataset_list:
        queryset_field_lte = dataset.get('dt_field') + '__lte'
        annual_monthly_counts = [dataset.get('queryset').filter(**{queryset_field_lte: ts}).count()
                                 for ts in annual_timestamp_list]

        return_dataset_list.append(
            dict(title=dataset.get('title'),
                 url=dataset.get('list_view'),
                 dt_field=dataset.get('dt_field'),
                 data=annual_monthly_counts,
                 color=color_list[color],
                 list_view=dataset.get('list_view'),
                 icon=dataset.get('icon'),
                 )
        )

        if color >= len(color_list) - 1:
            color = 0
        color += 1
    return month_labels, annual_timestamp_list, return_dataset_list


def build_annual_trend_chart(dataset_list):
    """
    process a list of datasets and return data required to plot an annual trend chart

    Args:
        dataset_list - (list of dictionaries) set of data to display; list of dictionaries containing the following:
                       title     - title to display for dataset
                       queryset  - queryset to use in counts
                       dt_field  - datetime field in model

                       example:
                           [{title='Owners', queryset=Owner.objects.all(), dt_field='created_at'}, ...]

    Returns:
        tuple containing:
            list of month labels
            list of month timestamps
            list of dictionaries containing data needed to plot chart
    """
    return_dataset_list = list()
    color_list = get_color_list()
    annual_timestamp_list = get_annual_timestamps(reverse=True)
    month_labels = [ts.strftime('%B') for ts in annual_timestamp_list]

    color = 0
    for dataset in dataset_list:
        data_year = dataset.get('queryset').filter(**{dataset.get('dt_field') + '__gte':
                                                          timezone.now() - datetime.timedelta(days=365.2425)})
        annual_monthly_counts = [
            len([i for i in data_year if getattr(i, dataset.get('dt_field')).month == ts.month and
                 getattr(i, dataset.get('dt_field')).year == ts.year])
            for ts in annual_timestamp_list
        ]

        return_dataset_list.append(
            dict(title=dataset.get('title'),
                 list_view=dataset.get('list_view'),
                 dt_field=dataset.get('dt_field'),
                 color=color_list[color],
                 annual=annual_monthly_counts,
                 )
        )

        if color >= len(color_list) - 1:
            color = 0
        color += 1
    return month_labels, annual_timestamp_list, return_dataset_list


def build_day_week_month_year_charts(dataset_list):
    """
    process a list of datasets and return data required to plot charts for counts per past day, week, month, and year

    Args:
        dataset_list - (list of dictionaries) set of data to display; list of dictionaries containing the following:
                       title     - title to display for dataset
                       queryset  - queryset to use in counts
                       dt_field  - datetime field in model
                       list_view - list view of data (used in links)

                       example:
                           [{title='Owners', queryset=Owner.objects.all(),
                             dt_field='created_at', list_view='/hostmgr/list_owners}, ...]

    Returns:
        list of dictionaries containing data needed to plot chart(s)
    """
    return_dataset_list = list()
    color_list = get_color_list()

    color = 0
    for dataset in dataset_list:
        data_day, data_week, data_month, data_year = get_dated_data(
            dataset.get('queryset'),
            dataset.get('dt_field') + '__gte',
        )

        return_dataset_list.append(
            dict(title=dataset.get('title'),
                 list_view=dataset.get('list_view'),
                 dt_field=dataset.get('dt_field'),
                 color=color_list[color],
                 day=data_day.count(),
                 week=data_week.count(),
                 month=data_month.count(),
                 year=data_year.count(),
                 ),
        )
        color += 1
        if color > len(color_list):
            color = 0
    return return_dataset_list


class AnnualStatView(View):
    """
    Description:
        Tallies the number of entries added over the past year. Included are counts of entries sectioned by
        past day, week, month and year.

    Parameters:
        title         - title displayed on web page
        subtitle      - subtitle displayed on web page
        template_name - template to use in rendering
        dataset_list  - set of data to display; list of dictionaries containing the following:
                            title     - title to display for dataset
                            queryset  - queryset to use in counts
                            dt_field  - datetime field in model
                            icon      - fontawesome icon to display for dataset
                            list_view - list view of data (used in links)

                        example:
                            [{title='Owners', queryset=Owner.objects.all(), dt_field='created_at',
                              icon='fas fa-users', list_view='/hostmgr/list_owners'), }, ...]
    """
    base_template = getattr(settings, 'BASE_TEMPLATE', 'handyhelpers/handyhelpers_base.htm')
    title = 'Annual Statistics Report'
    sub_title = None
    template_name = 'handyhelpers/report/annual_stats.html'
    dataset_list = list()

    def get(self, request):
        context = dict()
        context['base_template'] = self.base_template
        context['title'] = self.title
        context['sub_title'] = self.sub_title
        context['dataset_list'] = []
        last_day, last_week, last_month, last_year = get_timestamps()

        for dataset in self.dataset_list:
            data_day, data_week, data_month, data_year = get_dated_data(
                dataset.get('queryset'),
                dataset.get('dt_field') + '__gte',
            )

            context['dataset_list'].append(
                dict(title=dataset.get('title'),
                     icon=dataset.get('icon'),
                     url=dataset.get('list_view'),
                     total=dataset.get('queryset').count(),
                     day_count=data_day.count(),
                     day_date=last_day,
                     week_count=data_week.count(),
                     week_date=last_week,
                     month_count=data_month.count(),
                     month_date=last_month,
                     year_count=data_year.count(),
                     year_date=last_year,
                     ),
            )
        return render(request, self.template_name, context)


class AnnualTrendView(View):
    """
    Description:
        Tallies the number of elements processed over the past year. Included are counts of elements processed in the
        past day, week, month and year. Also includes counts of elements processed by month for the past year.

    Parameters:
        title         - title displayed on web page
        subtitle      - subtitle displayed on web page
        template_name - template to use in rendering
        dataset_list  - set of data to display; list of dictionaries containing the following:
                            title     - title to display for dataset
                            queryset  - queryset to use in counts
                            dt_field  - datetime field in model

                        example:
                            [{title='Owners', queryset=Owner.objects.all(), dt_field='created_at'}, ...]
    """
    base_template = getattr(settings, 'BASE_TEMPLATE', 'handyhelpers/handyhelpers_base.htm')
    title = 'Annual Trend Report'
    sub_title = 'data added over the past year'
    template_name = 'handyhelpers/report/annual_trends.html'
    chart_display_title = True
    chart_display_legend = False
    dataset_list = list()

    def get(self, request):
        context = dict()
        context['base_template'] = self.base_template
        context['title'] = self.title
        context['sub_title'] = self.sub_title
        context['dataset_list'] = list()
        context['last_day'], context['last_week'], context['last_month'], context['last_year'] = get_timestamps()
        context['month_labels'], context['month_timestamps'], context['annual_trend_dataset_list'] =  \
            build_annual_trend_chart(self.dataset_list)
        context['dataset_list'] = build_day_week_month_year_charts(self.dataset_list)
        context['chart_display_title'] = self.chart_display_title
        context['chart_display_legend'] = self.chart_display_legend
        return render(request, self.template_name, context)


class AnnualProgressView(View):
    """
    Description:
        Show the current number of elements per dataset and chart showing counts of data added per month over the
        past year per dataset.

    Parameters:
        title         - title displayed on web page
        subtitle      - subtitle displayed on web page
        template_name - template to use in rendering
        dataset_list  - set of data to display; list of dictionaries containing the following:
                            title     - title to display for dataset
                            queryset  - queryset to use in counts
                            dt_field  - datetime field in model

                        example:
                            [{title='Owners', queryset=Owner.objects.all(), dt_field='created_at'}, ...]
    """
    base_template = getattr(settings, 'BASE_TEMPLATE', 'handyhelpers/handyhelpers_base.htm')
    title = 'Annual Progress Report'
    sub_title = 'cumulative data added over the past year'
    template_name = 'handyhelpers/report/annual_progress.html'
    dataset_list = list()

    def get(self, request):
        context = dict()
        context['base_template'] = self.base_template
        context['title'] = self.title
        context['sub_title'] = self.sub_title
        context['month_labels'], context['month_timestamps'], context['annual_progress_dataset_list'] = \
            build_annual_progress_chart(self.dataset_list)
        return render(request, self.template_name, context=context)



from django import forms

from .mixins.form_mixins import SetRequiredMixin


class SetRequiredModelForm(SetRequiredMixin, forms.ModelForm):
    """
    A form class to add functionality from the SetRequiredMixin to a ModelForm. This allows for form fields specified
    in the 'required' parameter to be required on the rendered form. This further allows for form fields specified
    in the 'not_required' parameter to be not be required on the rendered form. Both the 'required' and 'not_required'
    parameters should be provided as a list or set when used.

        example:

        class MyModelForm(SetRequiredModelForm):

            class Meta:
                model = MyModel
                widgets = {
                    'some_field': forms.TextInput(attrs={'class': 'form-control'}),
                    'some_other_field': forms.TextInput(attrs={'class': 'form-control'}),
                }
                required = ['some_field']
                not_required = ['some_other_field']
    """



from django.core.checks import Warning, register


@register()
def user_tags_depreciated(app_configs, **kwargs):
    errors = list()
    errors.append(
        Warning(
            "The 'user_tags' template tags in django-handyhelpers has moved to 'handyhelper_tags'. Starting in "
            "version 0.2 'user_tags' will be unavailable to templates.",
            hint='change {% load user_tags %} to {% load handyhelper_tags %} in applicable templates.',
            id='user_tags',
            obj='django-handyhelpers',
        )
    )
    return errors



from django.core.management.base import BaseCommand, CommandError
from django.apps import apps
from django.conf import settings
from jinja2 import Template
import os

__version__ = '0.0.1'


class Command(BaseCommand):
    help = "Generate unittest file, based on a jinja2 template, for a given app"

    def __init__(self):
        self.opts = None
        self.app = None
        self.model_list = None
        super(Command, self).__init__()

    def add_arguments(self, parser):
        """ define command arguments """
        parser.add_argument('app', type=str, help='name of the django app')
        parser.add_argument('--test_type', type=str, help='django construct to create tests for (model, view, viewset)')
        parser.add_argument('--template', type=str, help='path to Jinja template used to create unittest file')
        parser.add_argument('--output_path', type=str, default='.', help='path where files should be created')
        parser.add_argument('--output_file', type=str, default=None, help='fully qualified name of file to be created; '
                                                                          'only use this when generating one file')
        parser.add_argument('--model', action='store_true', help='generate unittest for all models in app')

    def handle(self, *args, **options):
        """ command entry point """
        if options['app'] not in settings.INSTALLED_APPS:
            raise CommandError(f"'{options['app']}' is not an available application in this project")

        self.opts = options
        self.app = options['app']
        self.get_model_list()

        # build model unittests
        if 'model' in options['test_type']:
            self.build_model_unittests(output_path=options['output_path'],
                                       output_file=options['output_file'],
                                       template_file=options['template'])
        else:
            print('only model test are currently supported')

    @staticmethod
    def get_model_field_names(model, exclude_list=()):
        """ return a list of field names for a given model """
        return [i.name for i in model._meta.fields if type(i).__name__ not in exclude_list]

    def get_model_list(self):
        """ return a list of all models in application """
        app = apps.get_app_config(self.app)
        self.model_list = sorted(list(app.get_models()), key=lambda x: x._meta.model_name)

    def build_model_unittests(self, output_path=None, output_file=None, template_file=None):
        """ build unittest file for models """
        if not template_file:
            template_file = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                                         'unittest_templates', 'model_crud_update_all_fields.jinja')

        if output_file:
            output_path = output_file
        elif not output_path:
            output_path = 'test_model_crud.py'
        elif os.path.isdir(output_path):
            output_path = f'{output_path}/test_model_crud.py'

        auditlog_models = []
        if apps.is_installed('auditlog'):
            try:
                from auditlog.registry import auditlog
                auditlog_models = auditlog.get_models()
            except ModuleNotFoundError:
                pass

        model_dict = {}
        for model in self.model_list:
            skip_field_list = ['id', 'created_at', 'updated_at']
            update_field_type_list = ['CharField', 'ForeignKey', 'TextField']
            model_dict[model] = {
                'fill_optional': sorted([i.name for i in model._meta.fields if i.blank is True and i.null is False and
                                         i.unique is not True and i.name not in skip_field_list]),
                'update_field_list': sorted([i for i in model._meta.fields if i.get_internal_type() in
                                             update_field_type_list and not i.name.endswith('_id') and
                                             i.name not in skip_field_list], key=lambda x: x.name),
            }
            if model in auditlog_models:
                model_dict[model]['auditlog'] = True

        data = dict(model_dict=model_dict)

        with open(template_file) as f:
            template = Template(f.read())
        file_text = template.render(data)

        with open(output_path, 'w') as f:
            f.write(file_text)
        self.stdout.write(self.style.SUCCESS(f"{output_path} generated!"))



import re

from django import template
from django.contrib.auth.models import Group
from django.utils.safestring import mark_safe

register = template.Library()


@register.filter(name='has_group')
def has_group(user, group_name):
    """
    {% if request.user|has_group:"mygroup" %}
        <p>User belongs to my group
    {% else %}
        <p>User doesn't belong to mygroup</p>
    {% endif %}
    """
    try:
        group = Group.objects.get(name=group_name)
        return group in user.groups.all()
    except:
        return None


@register.filter(name='in')
def inside(value, arg):
    try:
        return value in arg
    except:
        return None


@register.filter
def add(value, arg):
    if arg:
        return value + arg


@register.filter
def subtract(value, arg):
    if arg:
        return value - arg


@register.filter()
def nbsp(value):
    return mark_safe("&nbsp;".join(value.split(' ')))


@register.filter
def intcomma(value):
    if value:
        return "{:,}".format(value)
    else:
        return 0


@register.filter
def usd(value):
    if value:
        return "${0:,.2f}".format(value)
    else:
        return 0


@register.filter
def or_blank(value):
    """ return an empty string if value is None """
    if value:
        return value
    else:
        return ""


@register.filter
def or_zero(value):
    """ return a zero if value is None """
    if value:
        return value
    else:
        return 0


@register.filter(name='percentage')
def percentage(value):
    if not value:
        return ""
    try:
        return "{:4.2f}%".format(float(value)*100)
    except:
        return ""


@register.filter(name='title_underscore')
def field_name_to_label(value):
    value = value.replace('_', ' ')
    return value.title()


@register.filter(name='host_ip_address')
def host_ip_address(value):
    return value.split(':')[0]


@register.filter(name='inlist')
def inlist(value, value_list):
    """
    return True if value is in a 'list' of values

        usage:
            {% value|inlist:"val1,val2,val3" %}

    Args:
        value: (str) value to check
        value_list: (str) comma separated values

    Returns:
        True if value in value_list, otherwise False
    """
    return True if value in value_list else False


@register.filter(name='in_any_group')
def in_any_group(user, group_list):
    """
    return True if user is in at least one group defined in 'list'

        usage:
            {% if request.user|in_any_group:"admins,operators,users" %}

    Args:
        user: user object
        group_list: (str) comma separated values

    Returns:
        True if user is in at least one group defined in 'group_list' otherwise False
    """
    return any(group in [i.name for i in user.groups.all()] for group in group_list.split(','))


@register.filter(name='in_all_group')
def in_all_groups(user, group_list):
    """
    return True if user is in all groups defined in 'list'

        usage:
            {% if request.user|in_all_groups:"admins,operators,users" %}

    Args:
        user: user object
        group_list: (str) comma separated values

    Returns:
        True if user is in all groups defined in 'group_list' otherwise False
    """
    return set(group_list.split(',')).issubset([i.name for i in user.groups.all()])


@register.filter(name='get_filtered_pagination_link')
def get_filtered_pagination_link(page, query_params):
    """
    return the paginated link with (filter) query parameters included
        usage example:
            <a href="?page={{ page_obj.paginator.num_pages|get_filtered_pagination_link:request.META.QUERY_STRING }}">

    Args:
        page: page number (probably from paginator)
        query_params: request query string
    """
    query_params = str(query_params)
    match = re.search('^page=\d+(.*)$', query_params)
    if match:
        filter_string = match.groups()[0]
        if not filter_string:
            return page
        return f'{page}{filter_string}'
    else:
        return f'{page}&{query_params}'
    return page


@register.filter(name='get_groups')
def get_groups(user):
    """
    Args:
        user: django user object

    Returns:
        list of group names representing all groups user is a member of
    """
    try:
        return [group.name for group in user.groups.all()]
    except Exception:
        return []


@register.filter(name='in_any')
def in_any(l1, l2):
    """
    checks if there is any intersection of two lists
    Args:
        l1: list
        l2: list

    Returns:
        True if there is at least one intersection; otherwise False
    """
    return True if set(l1).intersection(l2.split(',')) else False


@register.filter(name='next')
def dj_iter(gen):
    """
    return the next value as yielded from a generator
    Args:
        gen: generator object

    Returns:
        value as yielded from generator
    """
    try:
        return next(gen)
    except StopIteration:
        return None



"""
Permissions classes used with DRF APIs
"""

from rest_framework.permissions import BasePermission, SAFE_METHODS


class IsAdminOrReadOnly(BasePermission):
    """ The request is authenticated as an admin, or is a read-only request. """
    def has_permission(self, request, view):
        return (
            request.method in SAFE_METHODS or
            request.user and
            request.user.is_staff
        )


class IsInAllGroups(BasePermission):
    """ 
    Description:
        Restrict access based on request method and user group; user must be in ALL required groups 
    
    Usage:
        put the following in your viewset:
            permission_classes = (IsInAllGroups,)
            required_groups = {'POST': ['site_admins'],
                               'PATCH': ['site_admins'],
                               'PUT': ['site_admins'],
                               'DELETE': ['site_admins', 'site_managers'],
                              }
    """
    def has_permission(self, request, view):
        required_groups_mapping = getattr(view, 'required_groups', {})
        required_groups = required_groups_mapping.get(request.method, [])
        if required_groups is None:
            return False
        return set(required_groups).issubset([i.name for i in request.user.groups.all()])


class IsInAnyGroup(BasePermission):
    """ 
    Description:
        Restrict access based on request method and user group; user can be in ANY required group 
    
    Usage:
        put the following in your viewset:
            permission_classes = (IsInAnyGroup,)
            required_groups = {'POST': ['site_admins'],
                               'PATCH': ['site_admins'],
                               'PUT': ['site_admins'],
                               'DELETE': ['site_admins', 'superusers'],
                              }
    """
    def has_permission(self, request, view):
        required_groups_mapping = getattr(view, 'required_groups', {})
        required_groups = required_groups_mapping.get(request.method, [])
        if required_groups is None:
            return False
        return any(group in [i.name for i in request.user.groups.all()] for group in required_groups)


class InAllGroups(BasePermission):
    """
    Description:
        Restrict access based on request method and user group; user must be in ALL required groups

    Usage:
        put the following in your viewset:
            permission_classes = (InAllGroups,)
            permission_dict = {'POST': ['site_admins'],
                               'PATCH': ['site_admins'],
                               'PUT': ['site_admins'],
                               'DELETE': ['site_admins', 'site_managers'],
                              }
    """

    def has_permission(self, request, view):
        if not hasattr(view, 'permission_dict'):
            return False
        if request.user.is_superuser:
            return True
        permission_dict_mapping = getattr(view, 'permission_dict', {})
        permission_group_list = permission_dict_mapping.get(request.method, [])

        # if method is not provided, deny operation
        if permission_group_list is None:
            return False

        # if method is specified, but the group list is empty, allow operation
        if permission_group_list == []:
            return True

        return set(permission_group_list).issubset([i.name for i in request.user.groups.all()])


class InAnyGroup(BasePermission):
    """
    Description:
        Restrict access based on request method and user group; user can be in ANY required group

    Usage:
        put the following in your viewset:
            permission_classes = (InAnyGroup,)
            permission_dict = {'POST': ['site_admins'],
                               'PATCH': ['site_admins'],
                               'PUT': ['site_admins'],
                               'DELETE': ['site_admins', 'superusers'],
                              }
    """

    def has_permission(self, request, view):
        if not hasattr(view, 'permission_dict'):
            return False
        if request.user.is_superuser:
            return True

        permission_dict_mapping = getattr(view, 'permission_dict', {})
        permission_group_list = permission_dict_mapping.get(request.method, None)

        # if method is not provided, deny operation
        if permission_group_list is None:
            return False

        # if method is specified, but the group list is empty, allow operation
        if permission_group_list == []:
            return True

        return any(group in [i.name for i in request.user.groups.all()] for group in permission_group_list)



from django.core.management.base import BaseCommand
from django.contrib.auth.models import User
from django.db.utils import IntegrityError
import string
import random


class Command(BaseCommand):
    help = 'Add a user'

    def __init__(self):
        self.opts = None
        super(Command, self).__init__()

    def add_arguments(self, parser):
        """ define command arguments """
        parser.add_argument('--username', type=str, help='name for new user')
        parser.add_argument('--password', type=str, help='password for new user')
        parser.add_argument('--email', type=str, default=None, help='email address for new user')
        parser.add_argument('--random', action='store_true', required=False, help='generate a random username')
        parser.add_argument('--prefix', type=str, default='', help='prefix to use when generating a random username')
        parser.add_argument('--suffix', type=str, default='', help='suffix to use when generating a random username')
        parser.add_argument('--length', type=int, default=8, help='length to use for random username')
        parser.add_argument('--override', action='store_true', required=False,
                            help='if superuser already exists, delete and create new superuser')

    @staticmethod
    def generate_random_username(prefix='', suffix='', length=8):
        """ generate a random username """
        prefix = str(prefix) if prefix else ''
        suffix = str(prefix) if suffix else ''
        chars = string.ascii_letters + string.digits
        return prefix + ''.join(random.choice(chars) for i in range(length - len(prefix))) + suffix

    def handle(self, *args, **options):
        """ command entry point """
        self.opts = options

        # exit without creating a new user entry if username is provided but password is not provided
        if self.opts['username'] and not self.opts['password']:
            print('New user NOT created. When specifying a username, please include also include a password.')
            return

        # exit without creating a new user entry if username and random are not provided but password is not provided
        if not self.opts['username'] and not self.opts['random'] and self.opts['password']:
            print('New user NOT created. Please provide a username OR add --random to generate a random username.')
            return

        # generate a random username with a provided password
        elif self.opts['random'] and self.opts['password'] and not self.opts['username']:
            self.opts['username'] = self.generate_random_username(prefix=self.opts['prefix'],
                                                                  suffix=self.opts['suffix'],
                                                                  length=self.opts['length'])

        # generate a random username and password (username == password)
        elif self.opts['random'] and not self.opts['password']:
            random_value = self.generate_random_username(prefix=self.opts['prefix'],
                                                         suffix=self.opts['suffix'],
                                                         length=self.opts['length'])
            self.opts['username'] = self.opts['password'] = random_value
            print('Generated a random username/password for you; (password == username).')

        # if not random, and not username and not password, use default values ('user')
        elif not self.opts['username'] and not self.opts['password']:
            self.opts['username'] = 'user'
            self.opts['password'] = 'user'

        # first check if user exists
        try:
            current_user = User.objects.get(username=self.opts['username'])
        except User.DoesNotExist:
            current_user = None

        if current_user:
            if self.opts['override']:
                User.objects.get(username=self.opts['username']).delete()
                print(f'''Deleted existing user {self.opts['username']} so new user could be created.''')
                new_user = User.objects.create_user(self.opts['username'], self.opts['email'], self.opts['password'])
                print(f'New user {new_user} created!')
            else:
                print(f'''A user with the username `{self.opts['username']}` already exists.''')

        else:
            try:
                new_user = User.objects.create_user(self.opts['username'], self.opts['email'], self.opts['password'])
                print(f'New user `{new_user}` created!')
            except IntegrityError:
                print(f'''A user with the username `{self.opts['username']}` already exists.''')



from django.core.management.base import BaseCommand
from django.contrib.auth.models import User
from django.db.utils import IntegrityError


class Command(BaseCommand):
    help = 'Add a superuser'

    def __init__(self):
        self.opts = None
        super(Command, self).__init__()

    def add_arguments(self, parser):
        """ define command arguments """
        parser.add_argument('--username', type=str, help='name for new superuser')
        parser.add_argument('--password', type=str, help='password for new superuser')
        parser.add_argument('--email', type=str, default=None, help='email address for new superuser')
        parser.add_argument('--override', action='store_true', required=False,
                            help='if superuser already exists, delete and create new superuser')

    def handle(self, *args, **options):
        """ command entry point """
        self.opts = options

        # username is provided, but password was not
        if self.opts['username'] and not self.opts['password']:
            print('A password was not provided; password will be the same as the provided username.')
            self.opts['password'] = self.opts['username']

        # username was not provided, but password was
        if not self.opts['username'] and self.opts['password']:
            print('New superuser NOT created. Please provide a username, or leave both username and password blank to '
                  'use default values.')
            return

        # neither username nor password were provided
        if not self.opts['username'] and not self.opts['password']:
            self.opts['username'] = 'admin'
            self.opts['password'] = 'admin'

        # first check if user exists
        try:
            current_user = User.objects.get(username=self.opts['username'])
        except User.DoesNotExist:
            current_user = None

        if current_user:
            if self.opts['override']:
                User.objects.get(username=self.opts['username']).delete()
                print(f'''Deleted existing superuser `{self.opts['username']}` so new superuser could be created.''')
                new_user = User.objects.create_superuser(self.opts['username'],
                                                         self.opts['email'],
                                                         self.opts['password'])
                print(f'New superuser `{new_user}` created!')
            else:
                print(f'''A superuser with the username `{self.opts['username']}` already exists''')

        else:
            try:
                new_user = User.objects.create_superuser(self.opts['username'],
                                                         self.opts['email'],
                                                         self.opts['password'])
                print(f'New superuser `{new_user}` created!')
            except IntegrityError:
                print(f'''A superuser with the username `{self.opts['username']}` already exists''')
